/*
 * Copyright (C) 2005 USAGI/WIDE Project.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the project nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE PROJECT OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
/*
 * Author:
 *	Hideaki YOSHIFUJI <yoshfuji@linux-ipv6.org>
 */

#include <config.h>
#include <includes.h>
#include <radvd.h>
#include <pathnames.h>
#include <stdint.h>
#include <net/if.h>

#include "pathnames.h"

static int set_interface_mtu(const char *iface, uint32_t mtu)
{
	FILE *fp;
	char spath[sizeof(PROC_SYS_IP6_IFMTU) + IF_NAMESIZE + 1];
	sprintf(spath, PROC_SYS_IP6_IFMTU, iface);

	fp = fopen(spath, "w");
	if (!fp) {
		log(LOG_ERR, "failed to set mtu (%u) for %s",
		    (unsigned int)mtu, iface);
		return -1;
	}
	fprintf(fp, "%u", (unsigned int)mtu);
	fclose(fp);

	return 0;
}

static int set_interface_curhlim(const char *iface, uint8_t hlim)
{
	FILE *fp;
	char spath[sizeof(PROC_SYS_IP6_IFHLIM) + IF_NAMESIZE + 1];
	sprintf(spath, PROC_SYS_IP6_IFHLIM, iface);

	fp = fopen(spath, "w");
	if (!fp) {
		log(LOG_ERR, "failed to set hop limit (%u) for %s",
		    (unsigned int)hlim, iface);
		return -1;
	}
	fprintf(fp, "%u", (unsigned int)hlim);
	fclose(fp);

	return 0;
}

static int set_interface_rtime(const char *iface, uint32_t rtime)
{
	FILE *fp;
	char spath[sizeof(PROC_SYS_IP6_IFRTIME) + IF_NAMESIZE + 1];
	sprintf(spath, PROC_SYS_IP6_IFRTIME, iface);

	fp = fopen(spath, "w");
	if (!fp) {
		log(LOG_ERR, "failed to set reachable time (%u) for %s",
		    (unsigned int)rtime, iface);
		return -1;
	}
	fprintf(fp, "%u", (unsigned int)rtime);
	fclose(fp);

	return 0;
}

int set_interface_variables(const  char *iface,
			    uint32_t mtu, uint8_t hlim, uint32_t rtime)
{
	int ret = 0;
	if (mtu) {
		ret = set_interface_mtu(iface, mtu);
		if (ret)
			return ret;
	}
	if (hlim) {
		ret = set_interface_curhlim(iface, hlim);
		if (ret)
			return ret;
	}
	if (rtime) {
		ret = set_interface_rtime(iface, rtime);
	}
	return ret;
}

